;-----------------------------------------
; CRT0BASE.S 
; Basic definitions for the compiled code.
; Registers are located at the end of the
; first page (=256 Bytes) of the M68HC11
;-----------------------------------------
	.module crt0base.s
	.area   DIRECT (ABS,PAG)
	.org  0
;
; Null-Ptr Destination (hope, that nobody writes more than 2 bytes here)
;
_os_null_ptr::
	.blkb 2

;-------------------------------------
; Moved pseudo registers for talker
;  Steven L. Barnicki, 2/7/98
;-------------------------------------
; 
; Pseudo regs
;
ZD0:: .blkb 2
ZD1:: .blkb 2
ZD2:: .blkb 2
ZD3:: .blkb 2
ZD4:: .blkb 2
ZD5:: .blkb 2
ZD6:: .blkb 2
ZD7:: .blkb 2
; move up so idle loop lives
	.org 0x0BB
ZD8:: .blkb 2
ZB0:: .blkb 1
ZB1:: .blkb 1
ZB2:: .blkb 1
ZB3:: .blkb 1
ZB4:: .blkb 1
;
; scratch area
;
ZXT:: .blkb 2
_os_direct_end::


;-----------------------------------
; Registers area
;   Steven L. Barnicki, 2/1/99
;-----------------------------------
.area   REGS (ABS)
	.org  0x1000
_porta::.blkb 1
	.blkb 1         ;reserved
_pioc:: .blkb 1
_portc::.blkb 1
_portb::.blkb 1
_portcl::.blkb 1
	.blkb 1         ;reserved 
_ddrc:: .blkb 1
_portd::.blkb 1
_ddrd:: .blkb 1
_porte::.blkb 1
_cforc::.blkb 1
_oc1m:: .blkb 1
_oc1d:: .blkb 1
_tcnt:: .blkw 1
_tic1:: .blkw 1
_tic2:: .blkw 1
_tic3:: .blkw 1
_toc1:: .blkw 1
_toc2:: .blkw 1
_toc3:: .blkw 1
_toc4:: .blkw 1
_ti4::  .blkw 1
_tctl1::.blkb 1
_tctl2::.blkb 1
_tmsk1::.blkb 1
_tflg1::.blkb 1
_tmsk2::.blkb 1
_tflg2::.blkb 1
_pactl::.blkb 1
_pacnt::.blkb 1
_spcr:: .blkb 1
_spsr:: .blkb 1
_spdr:: .blkb 1
_baud:: .blkb 1
_sccr1::.blkb 1
_sccr2::.blkb 1
_scsr:: .blkb 1
_scdr:: .blkb 1
_adctl::.blkb 1
_adr1:: .blkb 1
_adr2:: .blkb 1
_adr3:: .blkb 1
_adr4:: .blkb 1
_bprot::.blkb 1
_eprog::.blkb 1
	.blkb 2	;reserved
_option::.blkb 1
_coprst::.blkb 1
_pprog::.blkb 1
_hprio::.blkb 1
_init:: .blkb 1
_test1::.blkb 1
_config::.blkb 1

;------------------
; end of registers
;------------------

;-----------------------------------------------
; Added loader section for code segment ordering
;   Steven L. Barnicki, 2/4/99
;-----------------------------------------------

	.area _LOADER (REL)
	jmp _start


;----------------------------------------
; Added startup code for the briefcase
;  Steven L. Barnicki, 2/7/98
;----------------------------------------
	.area _CODE
;-----------------------------------------------------------
_start::
			ldx  #0x1000      ; set up _os_io_base for 68HC11
			stx  _os_io_base  ;  4  3  
; OPTION Register
;     Bit 7: Analog/Digital Conversion Charge pump (1 = on)
;     Bit 6: A/D and EEPROM clock select (0 = E-Clock)
;     Bit 5: IRQ Edge-Sensitive  (0 = default)
;     Bit 4: Delay (1 = default)
;     Bit 0-1: Watchdog Rate
;          Bits   Div.Factor
;          00     1
;          01     4
;          10    16
;          11    64
			ldaa 0x039,x            ;  get existing data
			oraa #0b10000000        ;  Turn on A/D, leave others alone
			staa 0x039,x            ;  save it
; now set up the stack
			lds     #__stack_begin-1   ; initialize stack pointer
			jsr     __build_irt_table  ; build the interrupt table

			ldx     #0                 ; clear the NULL pointer
			stx     *_os_null_ptr      

			clr     _os_sei_cnt        ; clear interrupt disable counter

			cli
			jsr     _main              ; main()
_exit::;

exit::   bra     exit
___main::
			rts   ; return from function

;---------------------------------------
; end startup code
;---------------------------------------
; put any global library routines here
;
;
;Wait routines of 1 sec and 1 ms
;
; prototypes
;   void wait1ms(void);   waits one millisecond
;   void wait(int msec);  waits for msec milliseconds
;
;
;Written by Steven L. Barnicki, Ph.D.
;       2/7/98, v1.0
;
	.globl _wait
	.globl _wait1ms

_wait::
	psha
	tpa
	psha
	pshx
	ldx *ZD0      ;wait ZD0 milliseconds
1$:     jsr _wait1ms
	dex
	bne 1$
	pulx
	pula
	tap
	pula
	rts

_wait1ms::
	psha    ;1 ms wait
	tpa
	psha
	pshx
	ldx #200
2$:     dex
	nop
	nop
	bne 2$
	pulx
	pula
	tap
	pula
	rts



;---------------------------------------
; end global library routines
;---------------------------------------

;-----------------------------------------
; required gcclib code
;-----------------------------------------
;
; signed divide: assumes numerator on stack and denominator on stack + 2
; returns quotient in D
;
	.area _CODE

___divhi3::
	tsx
	ldd   4,x
	bge   $1
	ldd   #0
	subd  4,x
$1:
	std   *ZXT
	ldd   2,x
	bge   $2
	ldd   #0
	subd  2,x
$2:
	xgdx
	ldd   *ZXT
	idiv
	stx   *ZXT
	tsx
	ldd   4,x
	clrb
	eora  2,x
	anda  #0x80
	beq   $3
	ldd   #0
	subd  *ZXT
	bra   $4
$3:
	ldd   *ZXT
$4:
	rts

;
; signed modulo divide: assumes numerator on stack and denominator on stack + 2
; returns remainder in D
;
___modhi3::
	tsx
	ldd   4,x
	bge   $5
	ldd   #0
	subd  4,x
$5:
	std   *ZXT
	ldd   2,x
	bge   $6
	ldd   #0
	subd  2,x
$6:
	xgdx
	ldd   *ZXT
	idiv
	std   *ZXT
	tsx
	ldd   4,x
	clrb
	eora  2,x
	anda  #0x80
	beq   $7
	ldd   #0
	subd  *ZXT
	bra   $8
$7:
	ldd   *ZXT
$8:
	rts

;
; signed multiply: assumes e1 in D and e2 on stack
; returns result (e1 * e2) in D
;
___mulhi3::
	pulx
	pshx
	pshx
	xgdx
	pshx
	tsx
	ldab  1,x
	ldaa  7,x
	mul
	std   *ZXT
	ldab  0,x 
	ldaa  7,x
	mul
	tba
	clrb
	addd  *ZXT
	std   *ZXT
	ldab  1,x
	ldaa  6,x
	mul
	tba
	clrb
	addd  *ZXT
	pulx
	pulx
	rts

;
; See aux-output.c module for more details on these routines
;
; __pushregs::
;  rts
;
; __pullregs::
;  rts

;
;  end crt0.s
;
